const fs = require('fs');
const path = require('path');

const settingsFilePath = path.join(process.env.APPDATA || (process.platform === 'darwin' ? process.env.HOME + '/Library/Application Support' : process.env.HOME + "/.config"), 'GeminiNative', 'settings.json');
const settingsDirPath = path.dirname(settingsFilePath);

const defaultSettings = {
  autoHideEnabled: true,
  reloadOnPasteEnabled: true,
  isAlwaysOnTop: true,
  isFrameVisible: false,
  launchOnStartup: false, // Added: Setting for launch on startup
};

let currentSettings = { ...defaultSettings };

function ensureSettingsDirExists() {
  if (!fs.existsSync(settingsDirPath)) {
    fs.mkdirSync(settingsDirPath, { recursive: true });
  }
}

function loadSettings() {
  ensureSettingsDirExists();
  try {
    if (fs.existsSync(settingsFilePath)) {
      const data = fs.readFileSync(settingsFilePath, 'utf-8');
      const loadedSettings = JSON.parse(data);
      currentSettings = { ...defaultSettings, ...loadedSettings };
    } else {
      // Save default settings if file doesn't exist
      saveSettings();
    }
  } catch (error) {
    console.error('Error loading settings, using defaults:', error);
    currentSettings = { ...defaultSettings };
  }
  return currentSettings;
}

function saveSettings() {
  ensureSettingsDirExists();
  try {
    const data = JSON.stringify(currentSettings, null, 2);
    fs.writeFileSync(settingsFilePath, data, 'utf-8');
  } catch (error) {
    console.error('Error saving settings:', error);
  }
}

// Initialize settings on load
loadSettings();

module.exports = {
  getSetting: (key) => {
    return currentSettings[key];
  },
  updateSetting: (key, value) => {
    if (currentSettings[key] !== value) {
      currentSettings[key] = value;
      saveSettings();
    }
  },
  getAllSettings: () => {
    return { ...currentSettings };
  }
};
